@@ -422,8 +422,10 @@
 (define (tmtex-next-line l) (list '!nextline))
 
 (define (tmtex-decode-length s)
+  ;; FIXME: should be completed
   (cond ((string-ends? s "fn") (string-replace s "fn" "em"))
 	((string-ends? s "spc") (string-replace s "spc" "em"))
+	((string-ends? s "par") (string-replace s "par" "\\columnwidth"))
 	(else s)))
 
 (define (tmtex-hspace l)
@@ -811,7 +813,13 @@
   (tmtex-function 'tmaction l))
 
 (define (tmtex-postscript l)
-  (list 'epsfig (string-append "file=" (force-string (car l)))))
+  (let* ((fig (list 'epsfig (string-append "file=" (force-string (car l)))))
+	 (hor (if (== (cadr l) "") "!" (tmtex-decode-length (cadr l))))
+	 (ver (if (== (caddr l) "") "!" (tmtex-decode-length (caddr l)))))
+    (if (or (string-starts? hor "*") (string-starts? hor "/")) (set! hor "!"))
+    (if (or (string-starts? ver "*") (string-starts? ver "/")) (set! ver "!"))
+    (if (and (== hor "!") (== ver "!")) fig
+	(list 'resizebox hor ver fig))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Titles of documents
